@@ -11,7 +11,7 @@ setClass("GAlignmentPairs",
     contains="List",
     representation(
         strandMode="integer",         # single integer (0L, 1L, or 2L)
-        NAMES="characterORNULL",      # R doesn't like @names !!
+        NAMES="character_OR_NULL",    # R doesn't like @names !!
         first="GAlignments",          # of length N, no names, no elt metadata
         last="GAlignments",           # of length N, no names, no elt metadata
         isProperPair="logical",       # of length N
@@ -105,7 +105,10 @@ setMethod("first", "GAlignmentPairs",
     {
         if (!isTRUEorFALSE(real.strand))
             stop("'real.strand' must be TRUE or FALSE")
-        ans <- setNames(x@first, names(x))
+        ans <- x@first
+        x_names <- names(x)
+        if (!is.null(x_names))
+            ans <- setNames(ans, x_names)
         if (real.strand && strandMode(x) != 1L)
             strand(ans) <- .first_strand(x, real.strand=TRUE)
         ans
@@ -121,7 +124,10 @@ setMethod("second", "GAlignmentPairs",
     {
         if (!isTRUEorFALSE(real.strand))
             stop("'real.strand' must be TRUE or FALSE")
-        ans <- setNames(x@last, names(x))
+        ans <- x@last
+        x_names <- names(x)
+        if (!is.null(x_names))
+            ans <- setNames(ans, x_names)
         if (real.strand && strandMode(x) != 2L)
             strand(ans) <- .last_strand(x, real.strand=TRUE)
         ans
@@ -216,12 +222,15 @@ setMethod("seqlevelsInUse", "GAlignmentPairs",
 )
 
 setReplaceMethod("seqinfo", "GAlignmentPairs",
-    function(x, new2old=NULL, force=FALSE, value)
+    function(x, new2old=NULL, force=FALSE,
+             pruning.mode=c("error", "coarse", "fine", "tidy"),
+             value)
     {
         if (!is(value, "Seqinfo"))
             stop("the supplied 'seqinfo' must be a Seqinfo object")
         dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
                                   new2old=new2old, force=force,
+                                  pruning.mode=pruning.mode,
                                   seqlevels(value))
         if (length(dangling_seqlevels) != 0L) {
             dropme_in_first <- seqnames(x@first) %in% dangling_seqlevels
@@ -585,8 +594,6 @@ shrinkByHalf <- function(x)
     ans
 }
 
-### FIXME: Behavior is currently undefined (and undocumented) when
-### strandMode(x) is 0. Fix this!
 setMethod("grglist", "GAlignmentPairs",
     function(x, use.names=TRUE, use.mcols=FALSE, drop.D.ranges=FALSE)
     {
@@ -599,12 +606,19 @@ setMethod("grglist", "GAlignmentPairs",
             stop("'mcols(x)' cannot have reserved column \"query.break\"")
         x_first <- x@first
         x_last <- x@last
-        if (strandMode(x) == 1L) {
+        if (strandMode(x) == 0L) {
+            x_first <- unstrand(x_first)
+            x_last <- unstrand(x_last)
+            x_unlisted <- c(x_first, x_last)
+        } else if (strandMode(x) == 1L) {
             x_last <- invertStrand(x@last)
             x_unlisted <- c(x_first, x_last)
         } else if (strandMode(x) == 2L) {
             x_first <- invertStrand(x@first)
             x_unlisted <- c(x_last, x_first)
+        } else {
+            ## Should never happen.
+            stop("unsupported strandMode: ", strandMode(x))
         }
         ## Not the same as doing 'unlist(x, use.names=FALSE)'.
         collate_subscript <-
