@@ -6,12 +6,13 @@
 setClass("GAlignments",
     contains="Vector",
     representation(
-        NAMES="characterORNULL",      # R doesn't like @names !!
+        NAMES="character_OR_NULL",    # R doesn't like @names !!
         seqnames="Rle",               # 'factor' Rle
         start="integer",              # POS field in SAM
         cigar="character",            # extended CIGAR (see SAM format specs)
         strand="Rle",                 # 'factor' Rle
-        #mismatches="characterORNULL", # see MD optional field in SAM format specs
+        #mismatches="character_OR_NULL", # see MD optional field in SAM format
+                                         #specs
         elementMetadata="DataFrame",
         seqinfo="Seqinfo"
     ),
@@ -139,27 +140,41 @@ setReplaceMethod("strand", "GAlignments",
     }
 )
 
-setReplaceMethod("seqinfo", "GAlignments",
-    function(x, new2old=NULL, force=FALSE, value)
-    {
-        if (!is(value, "Seqinfo"))
-            stop("the supplied 'seqinfo' must be a Seqinfo object")
-        dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
-                                  new2old=new2old, force=force,
-                                  seqlevels(value))
-        if (length(dangling_seqlevels) != 0L)
-            x <- x[!(seqnames(x) %in% dangling_seqlevels)]
-        old_seqinfo <- seqinfo(x)
-        x@seqnames <- GenomeInfoDb:::makeNewSeqnames(x,
-                                  new2old, seqlevels(value))
-        x@seqinfo <- value
-        geom_has_changed <- GenomeInfoDb:::sequenceGeometryHasChanged(
-                                  seqinfo(x), old_seqinfo, new2old=new2old)
-        if (any(geom_has_changed, na.rm=TRUE))
-            validObject(x)
-        x
+### Does NOT suppoprt pruning mode "fine". Pruning modes "coarse" and "tidy"
+### are equivalent on a GAlignments object.
+### FIXME: This repeats most of the code in
+###        GenomicRanges:::set_GenomicRanges_seqinfo!
+set_GAlignments_seqinfo <-
+    function(x, new2old=NULL, force=FALSE,
+             pruning.mode=c("error", "coarse", "fine", "tidy"),
+             value)
+{
+    pruning.mode <- match.arg(pruning.mode)
+    if (pruning.mode == "fine")
+        stop(wmsg("\"fine\" pruning mode not supported on ",
+                  class(x), " objects"))
+    if (!is(value, "Seqinfo"))
+        stop("the supplied 'seqinfo' must be a Seqinfo object")
+    dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
+                              new2old=new2old, force=force,
+                              pruning.mode=pruning.mode,
+                              seqlevels(value))
+    if (length(dangling_seqlevels) != 0L) {
+        ## Prune 'x'.
+        non_dangling_range <- !(seqnames(x) %in% dangling_seqlevels)
+        x <- x[non_dangling_range]
     }
-)
+    old_seqinfo <- seqinfo(x)
+    x@seqnames <- GenomeInfoDb:::makeNewSeqnames(x,
+                              new2old, seqlevels(value))
+    x@seqinfo <- value
+    geom_has_changed <- GenomeInfoDb:::sequenceGeometryHasChanged(
+                              seqinfo(x), old_seqinfo, new2old=new2old)
+    if (any(geom_has_changed, na.rm=TRUE))
+        validObject(x)
+    x
+}
+setReplaceMethod("seqinfo", "GAlignments", set_GAlignments_seqinfo)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -332,11 +347,14 @@ setMethod("update", "GAlignments",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Helper function used by higher level coercion functions.
+### Internal helper function used by higher level coercion functions.
+###
+### NOT exported.
 ###
 
 ### Names are propagated via 'x@partitioning' ('x' is a CompressedIRangesList).
-.CompressedIRangesListToGRangesList <- function(x, seqnames, strand, seqinfo)
+make_GRangesList_from_CompressedIRangesList <- function(x, seqnames, strand,
+                                                           seqinfo)
 {
     x_eltNROWS <- elementNROWS(x)
     seqnames <- rep.int(seqnames, x_eltNROWS)
@@ -396,8 +414,8 @@ setMethod("grglist", "GAlignments",
                          use.mcols=use.mcols,
                          order.as.in.query=order.as.in.query,
                          drop.D.ranges=drop.D.ranges)
-        .CompressedIRangesListToGRangesList(rgl, seqnames(x), strand(x),
-                                                 seqinfo(x))
+        make_GRangesList_from_CompressedIRangesList(rgl,
+                         seqnames(x), strand(x), seqinfo(x))
     }
 )
 
@@ -694,3 +712,56 @@ setMethod("c", "GAlignments",
     }
 )
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Comparing/ordering.
+###
+
+setMethod("pcompare", c("GAlignments", "GAlignments"),
+    function(x, y)
+    {
+        x <- granges(x, use.names=FALSE)
+        y <- granges(y, use.names=FALSE)
+        callGeneric(x, y)
+    }
+)
+
+setMethod("is.unsorted", "GAlignments",
+    function(x, na.rm=FALSE, strictly=FALSE, ...) 
+    {
+        x <- granges(x, use.names=FALSE)
+        callGeneric()
+    }
+)
+
+setMethod("order", "GAlignments",
+    function(..., na.last=TRUE, decreasing=FALSE,
+                  method=c("auto", "shell", "radix"))
+    {
+        args <- list(...)
+        order_args <- c(lapply(args, granges, use.names=FALSE),
+                        list(na.last=na.last, decreasing=decreasing,
+                             method=method))
+        do.call(order, order_args)
+    }
+)
+
+setMethod("sort", "GAlignments",
+    function(x, decreasing=FALSE, ...)
+    {
+        oo <- GenomicRanges:::order_GenomicRanges(x, decreasing=decreasing,
+                                                      ...)
+        extractROWS(x, oo)
+    }
+)
+
+setMethod("rank", "GAlignments",
+    function(x, na.last=TRUE,
+             ties.method=c("average", "first", "last", "random", "max", "min"),
+             ...)
+    {
+        x <- granges(x, use.names=FALSE)
+        callGeneric()
+    }
+)
+
